import argparse
import datetime
import os
import os.path
import shutil
import subprocess

import dateutil.tz
import slugify


def setup(dest):
	module_root = os.path.dirname(os.path.dirname(__file__))
	example_dir = os.path.join(module_root, 'example')

	if os.path.exists(dest):
		def iter_copy(src_dir, dest_dir):
			for item in os.listdir(src_dir):
				src_path = os.path.join(src_dir, item)
				dest_path = os.path.join(dest_dir, item)
				if os.path.exists(dest_path):
					continue
				if os.path.isdir(src_path):
					shutil.copytree(src_path, dest_path)
				else:
					shutil.copy(
						os.path.join(example_dir, item),
						os.path.join(dest, item)
					)
		iter_copy(example_dir, dest)
	else:
		shutil.copytree(example_dir, dest)
		subprocess.check_call(['python3', '-m', 'venv', os.path.join(dest, '.venv')])
		subprocess.check_call([
			os.path.join(dest, '.venv', 'bin', 'pip'),
			'install', '-r', os.path.join(dest, 'requirements.txt'),
		])


def new_page(title):
	path = os.path.join('pages', slugify.slugify(title) + '.md')
	if os.path.exists(path):
		print(path, 'already exists!')
		return
	with open(path, 'w+') as page_file:
		page_file.write('# %s\n\nPage body here' % title)


def new_post(title):
	path = os.path.join('posts', slugify.slugify(title) + '.md')
	if os.path.exists(path):
		print(path, 'already exists!')
		return
	now = datetime.datetime.now(dateutil.tz.tzlocal())
	pubdate = now.strftime('%Y-%m-%d %H:%M %Z')
	with open(path, 'w+') as post_file:
		post_file.write('# %s\npubdate: %s\n\nPost body here' % (title, pubdate))


def generate():
	subprocess.check_call(['python', 'run.py'])


def serve():
	try:
		subprocess.check_call(['sh', '-c', 'cd dist && python -m http.server'])
	except KeyboardInterrupt:
		pass


def main():
	parser = argparse.ArgumentParser('russell')
	cmd_subparsers = parser.add_subparsers(dest='command')

	setup_parser = cmd_subparsers.add_parser('setup')
	setup_parser.add_argument('dir')

	new_parser = cmd_subparsers.add_parser('new')
	new_subparsers = new_parser.add_subparsers(dest='new_type')

	new_page_parser = new_subparsers.add_parser('page')
	new_page_parser.add_argument('title')

	new_post_parser = new_subparsers.add_parser('post')
	new_post_parser.add_argument('title')

	generate_parser = cmd_subparsers.add_parser('generate')

	serve_parser = cmd_subparsers.add_parser('serve')

	args = parser.parse_args()
	if not args.command:
		return parser.print_help()

	if args.command == 'setup':
		return setup(args.dir)
	if args.command == 'new':
		if args.new_type == 'page':
			return new_page(args.title)
		if args.new_type == 'post':
			return new_post(args.title)
	if args.command == 'generate':
		return generate()
	if args.command == 'serve':
		return serve()


if __name__ == '__main__':
	main()



from xml.etree import ElementTree as etree
import re


def text_element(tag, text):
	elem = etree.Element(tag)
	elem.text = text
	return elem


class SitemapGenerator:
	def __init__(self, blog, https=True):
		self.blog = blog
		self.schema = 'https' if https else 'http'

	def normalize_url(self, url):
		return re.sub(r'^\/\/', self.schema + '://', url)

	def generate_sitemap(self):
		tree = etree.Element('urlset', xmlns='http://www.sitemaps.org/schemas/sitemap/0.9')
		tree.append(self.get_index_element(self.blog.root_url))
		for page in self.blog.pages:
			if page.public:
				tree.append(self.get_page_element(page))
		for tag in self.blog.tags:
			tree.append(self.get_tag_element(tag))
		for post in self.blog.get_posts():
			tree.append(self.get_post_element(post))
		return etree.tostring(tree, 'utf-8')

	def get_post_element(self, post):
		elem = etree.Element('url')
		elem.append(text_element('loc', self.normalize_url(post.url)))
		elem.append(text_element('lastmod', post.pubdate.strftime('%Y-%m-%d')))
		elem.append(text_element('changefreq', 'monthly'))
		return elem

	def get_page_element(self, page):
		elem = etree.Element('url')
		elem.append(text_element('loc', self.normalize_url(page.url)))
		elem.append(text_element('changefreq', 'monthly'))
		return elem

	def get_tag_element(self, tag):
		elem = etree.Element('url')
		elem.append(text_element('loc', self.normalize_url(tag.url)))
		elem.append(text_element('changefreq', 'weekly'))
		return elem

	def get_index_element(self, url):
		elem = etree.Element('url')
		elem.append(text_element('loc', self.normalize_url(url)))
		elem.append(text_element('changefreq', 'daily'))
		return elem


def generate_sitemap(blog, https=True):
	return SitemapGenerator(blog, https).generate_sitemap()



#!/usr/bin/env python3

import os.path
import logging
import sass
from russell import BlogEngine

ROOT_DIR = os.path.dirname(__file__)

logging.basicConfig()

blog = BlogEngine(
	root_path=ROOT_DIR,
	root_url='//localhost',
	site_title='Russell example',
	site_desc=('An example Russell site.')
)

# add content
blog.add_pages()
blog.add_posts()

# copy and generate assets
blog.copy_assets()
blog.write_file('assets/style.css', sass.compile(
	filename=os.path.join(ROOT_DIR, 'style.sass')
))
blog.add_asset_hashes()

# generate HTML pages
blog.generate_index(num_posts=3)
blog.generate_archive()
blog.generate_pages()
blog.generate_posts()
blog.generate_tags()

# generate other stuff
blog.generate_sitemap(https=False)
blog.generate_rss()
blog.write_file('robots.txt', 'User-agent: *\nDisallow:\n')



from datetime import datetime
import hashlib
import logging
import os
import os.path
import shutil

import jinja2
import PyRSS2Gen

import russell.content
import russell.sitemap

LOG = logging.getLogger(__name__)


def _listfiles(root_dir):
	results = set()

	for root, _, files in os.walk(root_dir):
		for file in files:
			results.add(os.path.join(root, file))

	return results


def _rss_item(post, only_excerpt=True):
	if only_excerpt:
		read_more = 'Read the full article at %s' % post.url
		body = '<p>%s</p><p>%s</p>' % (post.excerpt, read_more)
	else:
		body = post.body

	return PyRSS2Gen.RSSItem(
		title=post.title,
		description=body,
		link=post.url,
		pubDate=post.pubdate,
	)


def make_link(title, url, blank=None):
	attrs = 'href="{}"'.format(url)
	if blank:
		attrs += ' target="_blank" rel="noopener noreferrer"'
	return '<a {}>{}</a>'.format(attrs, title)


class BlogEngine:
	def __init__(self, root_path, root_url, site_title, site_desc=None):
		self.root_path = root_path
		self.root_url = root_url
		self.site_title = site_title
		self.site_desc = site_desc

		self.cm = russell.content.ContentManager(root_url)
		self.pages = self.cm.pages
		self.posts = self.cm.posts
		self.tags = self.cm.tags

		self.asset_hash = {}

		self.jinja = jinja2.Environment(
			loader=jinja2.FileSystemLoader(os.path.join(root_path, 'templates')),
		)
		self.jinja.globals.update({
			'a': make_link,
			'asset_hash': self.asset_hash,
			'asset_url': self.get_asset_url,
			'now': datetime.now(),
			'root_url': self.root_url,
			'site_description': self.site_desc,
			'site_title': self.site_title,
			'tags': self.tags,
		})

	def get_asset_url(self, path):
		url = self.root_url + '/assets/' + path
		if path in self.asset_hash:
			url += '?' + self.asset_hash[path]
		return url

	def add_pages(self, path='pages'):
		pages_path = os.path.join(self.root_path, path)
		pages = []
		for file in _listfiles(pages_path):
			page_dir = os.path.relpath(os.path.dirname(file), pages_path)
			if page_dir == '.':
				page_dir = None
			kwargs = {'dir': page_dir}
			pages.append(self.cm.Page.from_file(file, kwargs=kwargs))
		self.cm.add_pages(pages)

	def add_posts(self, path='posts'):
		path = os.path.join(self.root_path, path)
		self.cm.add_posts([
			self.cm.Post.from_file(file)
			for file in _listfiles(path)
		])

	def copy_assets(self, path='assets'):
		assets_path = os.path.abspath(path)
		for root, _, files in os.walk(assets_path):
			for file in files:
				fullpath = os.path.join(root, file)
				relpath = os.path.relpath(fullpath, assets_path)
				copy_to = os.path.join(self._get_dist_path(relpath, dir='assets'))
				LOG.debug('copying %r to %r', fullpath, copy_to)
				shutil.copyfile(fullpath, copy_to)

	def add_asset_hashes(self, path='dist/assets'):
		for fullpath in _listfiles(os.path.join(self.root_path, path)):
			relpath = fullpath.replace(self.root_path + '/' + path + '/', '')
			md5sum = hashlib.md5(open(fullpath, 'rb').read()).hexdigest()
			LOG.debug('MD5 of %s (%s): %s', fullpath, relpath, md5sum)
			self.asset_hash[relpath] = md5sum

	def get_posts(self, num=None, tag=None, private=False):
		posts = self.posts.copy()
		if not private:
			posts = [post for post in posts if post.public]
		if tag:
			posts = [post for post in posts if tag in post.tags]
		if num:
			return posts[:num]
		return posts

	def _get_dist_path(self, path, dir=None):
		if isinstance(path, str):
			path = [path]
		if dir:
			path.insert(0, dir)
		return os.path.join(self.root_path, 'dist', *path)

	def _get_template(self, template):
		if isinstance(template, str):
			template = self.jinja.get_template(template)
		return template

	def generate_pages(self):
		for page in self.pages:
			self.generate_page(page.slug, template='page.html.jinja', page=page)

	def generate_posts(self):
		for post in self.posts:
			self.generate_page(['posts', post.slug],
				template='post.html.jinja', post=post)

	def generate_tags(self):
		for tag in self.tags:
			posts = self.get_posts(tag=tag, private=True)
			self.generate_page(['tags', tag.slug],
				template='archive.html.jinja', posts=posts)

	def generate_page(self, path, template, page=None, **kwargs):
		page_dir = page.dir if page else None
		path = self._get_dist_path(path, dir=page_dir)
		if not path.endswith('.html'):
			path = path + '.html'
		if not os.path.isdir(os.path.dirname(path)):
			os.makedirs(os.path.dirname(path))
		template = self._get_template(template)
		with open(path, 'w+') as file:
			file.write(template.render(page=page, **kwargs))

	def generate_index(self, num_posts=5):
		posts = self.get_posts(num=num_posts)
		self.generate_page('index', template='index.html.jinja', posts=posts)

	def generate_archive(self):
		self.generate_page('archive', template='archive.html.jinja', posts=self.get_posts())

	def generate_rss(self, path='rss.xml', only_excerpt=True):
		items = (
			_rss_item(post, only_excerpt=only_excerpt)
			for post in self.get_posts()
		)

		rss = PyRSS2Gen.RSS2(
			title=self.site_title,
			description=self.site_desc or '',
			link=self.root_url,
			lastBuildDate=datetime.now(),
			items=items,
		)

		path = self._get_dist_path(path)
		if not os.path.isdir(os.path.dirname(path)):
			os.makedirs(os.path.dirname(path))
		with open(path, 'w+') as file:
			rss.write_xml(file)

	def generate_sitemap(self, https=False):
		sitemap = russell.sitemap.generate_sitemap(self, https=https)
		self.write_file('sitemap.xml', sitemap)

	def write_file(self, path, contents):
		path = self._get_dist_path(path)
		if not os.path.isdir(os.path.dirname(path)):
			os.makedirs(os.path.dirname(path))
		if isinstance(contents, bytes):
			mode = 'wb+'
		else:
			mode = 'w'
		with open(path, mode) as file:
			file.write(contents)



#!/usr/bin/env python3

from __future__ import print_function
from setuptools import setup, find_packages
import os.path
import sys

if sys.version_info[0] != 3:
    print("Only Python 3 is supported!")
    sys.exit(1)

def read_requirements(path='requirements.txt'):
    fullpath = os.path.join(os.path.dirname(__file__), path)
    with open(fullpath, 'r') as f:
        lines = f.readlines()
    lines = [l.strip() for l in lines if l]
    return [l for l in lines if l]

setup(
    name='russell',
    packages=['russell'],
    version='0.5.3',
    license='MIT',
    description='A static HTML blog generator.',
    author='Andreas Lutro',
    author_email='anlutro@gmail.com',
    url='https://github.com/anlutro/russell',
    keywords=['blog', 'static', 'html', 'generator'],
    classifiers=[],
    install_requires=read_requirements(),
    entry_points={
        'console_scripts': [
            'russell=russell.cli:main'
        ],
    },
)



from datetime import datetime
import logging
import os.path

import time
import dateutil.parser
import dateutil.tz
import markdown
import slugify

LOG = logging.getLogger(__name__)
SYSTEM_TZINFO = dateutil.tz.tzlocal()


def _get_excerpt(body):
	excerpt_parts = []
	# iterate through lines until we find an empty line/two newlines in a row
	for line in body.splitlines():
		if line == '':
			break
		excerpt_parts.append(line)
	return ' '.join(excerpt_parts)


def _get_description(body, max_chars=100, search='. '):
	stop_idx = body.rfind(search, 0, max_chars)
	if stop_idx == -1 or stop_idx < max_chars:
		stop_idx = body.find(search, max_chars)
	if stop_idx == -1:
		return body
	return body[0:stop_idx+1]


def _parse_pubdate(pubdate):
	return dateutil.parser.parse(pubdate)


def _str_to_bool(string):
	norm_string = str(string).strip().lower()
	if norm_string in ('yes', 'true'):
		return True
	elif norm_string in ('no', 'false', ''):
		return False
	raise ValueError('Invalid boolean string: {}'.format(repr(string)))


class Content:
	cm = None

	@property
	def root_url(self):
		if self.cm:
			return self.cm.root_url
		return '//localhost'


class Entry(Content):
	def __init__(self, title, body, slug=None, subtitle=None, description=None, public=True):
		self.title = title
		self.body = body
		self.slug = slug or slugify.slugify(title)
		self.subtitle = subtitle
		self.description = description
		self.public = public

	@property
	def url(self):
		return self.root_url + '/' + self.slug

	@classmethod
	def from_string(cls, contents, kwargs=None):
		if kwargs is None:
			kwargs = {}

		lines = contents.splitlines()
		title = None
		description = None

		line = lines.pop(0)
		while line != '':
			if not title and line.startswith('#'):
				title = line[1:].strip()
			elif line.startswith('title:'):
				title = line[6:].strip()
			elif line.startswith('description:'):
				description = line[12:].strip()
			elif line.startswith('subtitle:'):
				kwargs['subtitle'] = line[9:].strip()
			elif line.startswith('comments:'):
				try:
					kwargs['allow_comments'] = _str_to_bool(line[9:])
				except ValueError:
					LOG.warning('invalid boolean value for comments', exc_info=True)

			cls.process_meta(line, kwargs)

			line = lines.pop(0)

		# the only lines left should be the actual contents
		body = '\n'.join(lines).strip()
		excerpt = _get_excerpt(body)
		if description is None:
			description = _get_description(excerpt, 160)
		if issubclass(cls, Post):
			kwargs['excerpt'] = markdown.markdown(excerpt)
		body = markdown.markdown(body)

		return cls(title=title, body=body, description=description, **kwargs)

	@classmethod
	def process_meta(cls, line, kwargs):
		if line.startswith('slug:'):
			kwargs['slug'] = line[5:].strip()

		elif line.startswith('public:'):
			try:
				kwargs['public'] = _str_to_bool(line[7:])
			except ValueError:
				LOG.warning('invalid boolean value for public', exc_info=True)

		elif line.startswith('private:'):
			try:
				kwargs['public'] = not _str_to_bool(line[8:])
			except ValueError:
				LOG.warning('invalid boolean value for private', exc_info=True)

	@classmethod
	def from_file(cls, path, kwargs=None):
		if kwargs is None:
			kwargs = {}

		LOG.debug('creating %s from "%s"', cls, path)

		# the filename will be the default slug - can be overridden later
		kwargs['slug'] = os.path.splitext(os.path.basename(path))[0]

		# if a pubdate wasn't found, use the file's last modified time
		if issubclass(cls, Post) and not kwargs.get('pubdate'):
			timestamp = min(os.path.getctime(path), os.path.getmtime(path))
			kwargs['pubdate'] = datetime.fromtimestamp(timestamp)

		with open(path, 'r') as file:
			entry = cls.from_string(file.read(), kwargs)

		return entry

	def __lt__(self, other):
		return self.title < other.title


class Page(Entry):
	def __init__(self, *args, allow_comments=False, dir=None, **kwargs):
		super().__init__(*args, **kwargs)
		self.allow_comments = allow_comments
		self.dir = dir


class Post(Entry):
	def __init__(self, *args, pubdate=None, excerpt=None, tags=None, allow_comments=True, **kwargs):
		super().__init__(*args, **kwargs)
		self.excerpt = excerpt or _get_excerpt(self.body)
		self.pubdate = pubdate
		self.tags = tags or []
		self.allow_comments = allow_comments

	@classmethod
	def make_tag(cls, tag_name):
		if cls.cm:
			return cls.cm.make_tag(tag_name)
		return Tag(tag_name.strip())

	@classmethod
	def process_meta(cls, line, kwargs):
		super().process_meta(line, kwargs)

		if line.startswith('pubdate:'):
			pubdate_str = line[8:].strip()
			try:
				kwargs['pubdate'] = _parse_pubdate(pubdate_str)
			except ValueError:
				LOG.warning('invalid pubdate given', exc_info=True)
			if 'pubdate' in kwargs and not kwargs['pubdate'].tzinfo:
				kwargs['pubdate'] = kwargs['pubdate'].replace(tzinfo=SYSTEM_TZINFO)
				LOG.warning('found pubdate without timezone: %r - using %r',
					pubdate_str, SYSTEM_TZINFO)

		elif line.startswith('tags:'):
			line_tags = line[5:].strip().split(',')
			kwargs['tags'] = [cls.make_tag(tag) for tag in line_tags if tag]

	@property
	def url(self):
		return self.root_url + '/posts/' + self.slug

	@property
	def tag_links(self):
		return ['<a href="' + tag.url + '">' + tag.title + '</a>' for tag in self.tags]

	def __lt__(self, other):
		if self.pubdate == other.pubdate:
			return self.title < other.title
		return self.pubdate > other.pubdate


class Tag(Content):
	def __init__(self, title, slug=None):
		if not title:
			raise ValueError('cannot create Tag with empty title')
		self.title = title
		self.slug = slug or slugify.slugify(title)

	@property
	def url(self):
		return self.root_url + '/tags/' + self.slug

	def __lt__(self, other):
		return self.title < other.title


class CaseInsensitiveDict(dict):
	def __setitem__(self, key, value):
		super().__setitem__(key.lower(), value)

	def __getitem__(self, key):
		return super().__getitem__(key.lower())

	def __contains__(self, key):
		return super().__contains__(key.lower())


class ContentManager:
	def __init__(self, root_url):
		self.Page = type('CM_Page', (Page,), {'cm': self})
		self.Post = type('CM_Post', (Post,), {'cm': self})
		self.Tag = type('CM_Tag', (Tag,), {'cm': self})
		self.root_url = root_url
		self.pages = []
		self.posts = []
		self.tags = []
		self.tags_dict = CaseInsensitiveDict()

	def make_tag(self, tag_name):
		tag_name = tag_name.strip()
		if tag_name not in self.tags_dict:
			self.tags_dict[tag_name] = self.Tag(tag_name)
		return self.tags_dict[tag_name]

	def add_pages(self, pages, resort=True):
		self.pages.extend(pages)
		if resort:
			self.pages.sort()

	def add_posts(self, posts, resort=True):
		self.posts.extend(posts)
		for post in posts:
			for tag in post.tags:
				if tag not in self.tags:
					self.tags.append(tag)

		if resort:
			self.tags.sort()
			self.posts.sort()
